import std/json, std/options, std/strutils, std/times

import ./claims, ./joseheader, ./jwa, ./jwe, ./jwk, ./jws, ./jwssig

export claims

type
  InvalidToken* = object of CatchableError

  JwtKind = enum jkJWE, jkJWS

  JWT* = object
    claims: JwtClaims
    case kind: JwtKind
    of jkJWE: jwe: JWE
    of jkJWS: jws: JWS

## Public read-only accessors to JWT members
## -----------------------------------------

func claims*(jwt: JWT): JwtClaims = jwt.claims

func header*(jwt: JWT): JoseHeader =
  case jwt.kind:
    of jkJWS: return jwt.jws[0].protected
    of jkJWE: return jwt.jwe.header

func `$`*(jwt: JWT): string =
  case jwt.kind:
    of jkJWS:
        result = jwt.header.rawB64 & "." & jwt.claims.rawB64 & "."
        if jwt.jws.len > 0: result &= jwt.jws[0].signatureB64
    of jkJWE:
        result = jwt.header.rawB64 & "." & jwt.claims.rawB64 & "."

proc initJWT*(encoded: string): JWT =
  let parts = encoded.split('.')

  if parts.len == 3:
    let jws = initJWS(encoded)
    result = JWT(
      kind: jkJWS,
      claims: initJwtClaims(jws.payloadB64),
      jws: jws)
  else:
    # TODO
    raise newException(Exception, "not yet implemented")

proc toJWT*(encoded: string): JWT = initJWT(encoded)

proc createSignedJwt*(
    header: JoseHeader,
    claims: JwtClaims,
    key: JWK
  ): JWT =
  ## Create a new JWT with the given header, claims, and signed with the given
  ## key.

  result = JWT(
    kind: jkJWS,
    claims: claims,
    jws: sign(claims, header, key))

proc validate*(jwt: JWT, sigAlg: JwtAlgorithm, key: JWK) =
  case jwt.kind:
    of jkJWS:
      jwt.jws.validate(sigAlg, key)

    of jkJWE:
      raise newException(Exception, "JWE validation is not yet implemented")

proc tryValidate*(jwt: JWT, sigAlg: JwtAlgorithm, key: JWK): bool =
  try:
    jwt.validate(sigAlg, key)
    return true
  except: return false

# proc verify*(jwt: JWT): bool =

# proc sign*(jwt: JWT, key: JWK): JWT =
